module Extract where

import CTrie
import Tok
import Paradigm
import AbsExtract
import MatchReg
import IO
import Monad
import UTF8
import qualified Data.Set as Set 
import List (isSuffixOf,intersperse)
import Data.Char (isSpace,isDigit)
import Text.Printf(hPrintf)

delimit = "\t"

ffind :: Options -> Paradigms -> FilePath -> IO ()
ffind opts ps file = do
  hPutStr stderr $ "Read " ++ (show (length ps)) ++ " paradigm(s).\n"
  s  <- readFile file
  case word_list s of
   xs -> do buildTrieWordlist xs (elem "-r" opts)
            process (Set.fromList xs) (Set.empty) (0,0)
            -- mapM_ putStrLn (Set.toList output_set) 
 where
  word_list s = tokens (["-" ++ o | _:o <- opts] ++ 
		   ["-nopunct","-nonumber"]) s
  process set output_set p@(hit,miss)
   | Set.null set = do let f  x = ((fromInteger (toInteger x)) :: Float)
                           s1 = f hit
			   s2 = f miss
                       hPrintf stderr "\nUsed   : %.2e" s1
		       hPrintf stderr "\nUnused : %.2e\n"  s2
   | otherwise = 
      do let (x,newset) = Set.deleteFindMin set
         -- ev <- 
         (b,output) <- printLex opts (find ps x)
         case output of
          (_:_) | elem "-u" opts -> 
                          do os <- foldM process_set output_set output
                             process newset os (update b p)
          (_:_) -> do sequence_ $ map print_it output
                      process newset output_set (update b p)
          _     -> process newset output_set (update b p)
  process_set set x
   | Set.member x set = return set
   | otherwise        = do print_it x
                           return $ Set.insert x set
  update b (i1,i2)
   | b         = (i1+1,i2)
   | otherwise = (i1,i2+1)
  print_it x
   | elem "-utf8" opts = putStrLn $ encodeUTF8 x
   | otherwise         = putStrLn x

printLex :: [String] -> (String, [(Name, String, [String], [String])]) -> IO (Bool,[String])
printLex opts (w,ps) = case ps of
  [] | elem "-nobad" opts -> return (False,[]) --[]
{-
  _  | elem "-gf" opts -> do
    let (a,c) = unzip [("fun " ++ f ++ " : " ++ cat ++ " ;",
                        "lin " ++ f ++ " = " ++ 
                            o ++ " " ++ printArgs (ss) ++ " ;") 
                                   | ((p,ss),i) <- zip ps [1..],
                                     let (cat,o) = pname2cat p,
                                     let f = w ++ show i]
    mapM_ putStrLn a
    mapM_ putStrLn c
-}
  [] -> do return $ (False,["-- " ++ w])
           -- return []
  _ | elem "-e" opts -> 
    do return (True,[p ++ " " ++ unwords ss ++ delimit ++ printEvidence es ++ pr_id opts pid | (p,pid, ss, es) <- ps])
       -- return $ concat $ [es | (_,_,es) <- ps]
  _ -> do
        return (True,[p ++ " " ++ unwords ss ++ pr_id opts pid | (p,pid,ss, _) <- ps])
        -- return $ concat $ [es | (_,_,es) <- ps]

pr_id :: [String] -> String -> String
pr_id opts pid 
 | elem "-id" opts = delimit ++ pid
 | otherwise       = ""

printEvidence :: [String] -> String
printEvidence [] = []
printEvidence (x:xs) = "(" ++ concat (intersperse "," (x:xs)) ++ ")"

printArgs :: [String] -> String
printArgs = unwords . map (\s -> '"':s++"\"")

find :: Paradigms -> String -> (String,[(Name,String, [String],[String])])
find ps w = (w, matchParadigms ps w)

matchParadigms :: Paradigms ->  String -> [(Name,String,[String], [String])]
matchParadigms ps w = mParadigms ps
 where
  mParadigms [] = []
  mParadigms (p:ps) = 
    case positive_match p w of     
     [] -> mParadigms ps
     xs -> [(name p, p_id p, [generateString (replaceVar r env) | r <- args p], evidence) 
                              | env <- xs,
	                        Just evidence <- [foundInTrie (constraints p) env]] ++ mParadigms ps
--if (foundInTrie tr (constraints p) xs) then
--	      (name p,  [generateString (replaceVar r xs) | r <- args p]) : mParadigms ps
--            else mParadigms ps

foundInTrie :: RegLogic -> [(Ident,String)] -> Maybe [String]
foundInTrie (RConj l1 l2) env = do xs <- foundInTrie l1 env
                                   ys <- foundInTrie l2 env
                                   return $ xs ++ ys
foundInTrie (RDisj l1 l2) env = case (foundInTrie l1 env,
				         foundInTrie l2 env) of
                                    (Just xs, Just ys) -> return $ xs ++ ys
                                    (xs, Nothing) -> xs
                                    (Nothing, xs) -> xs
foundInTrie (RNeg l1) env = 
    case (foundInTrie l1 env) of
     Just _ -> Nothing
     Nothing -> Just []
foundInTrie (RL reg) env = 
    let r1 = replaceVar reg env
	s  = generateString r1
     in if isInTrie s then
          return [s]
         else Nothing
			      
pname2cat :: Name -> (String,String)
pname2cat n = case span (/='!') n of
  (c,s) -> (c,drop 1 s)
