module Main where

import Paradigm
import Extract
import System
import ErrM
import List (partition, (\\))
import IO

main :: IO ()
main = do
  xs    <- getArgs
  case check_options xs of
   Ok _ ->
    if (elem "-h" xs) then do prErr welcome
                              prErr help_message
     else 
      do
       let (opts,fs) = partition ((=='-') . head) xs
       case fs of
        [parafile,corpus] -> do
         result <- get_paradigms parafile (elem "-utf8" xs)
	 case result of
	  Bad s  -> do prErr welcome
                       prErr s
          Ok  ps -> do prErr welcome
                       ffind opts ps corpus
        _ -> do prErr welcome
                prErr help_message
   Bad s -> do prErr welcome
               prErr s

welcome :: String
welcome = unlines
	    [
	     "",
	     "********************************************",
	     "*            The Extract Tool              *",
	     "********************************************",
	     "* v1.0Beta4                                *",
	     "* (c) Markus Forsberg & Aarne Ranta 2006   *",
	     "* under GNU General Public License.        *",
	     "********************************************",
	     ""
	    ]

help_message :: String
help_message =
    unlines 
    [" Help message:                                   ",
     " extract [Option(s)] paradigm_file  Corpus_File  ", 
     " Options:                                        ",
     "  -h       Display this message                  ",
     "  -utf8    Use UTF-8 encoding                    ",
     "  -nobad   Keep only the analysed words          ",
     "  -uncap   Transform uppercase to lowercase      ",
     "  -nocap   Remove all words in uppercase         ",
     "  -e       Print evidence as comment             ",
     "  -u       Print no duplicates                   ",
     "  -id      Print paradigm identifier             ",
     "  -r       Reverse words                         "
    ]

check_options xs = case filter (\s -> head s == '-') (xs \\ options) of
		    [] -> return ()
		    xs -> Bad $ "Invalid options: " ++ unwords xs

options = ["-utf8","-h", "-nobad", "-uncap", "-nocap", "-e", "-u","-id", "-r"]

prErr s =  hPutStr stderr (s ++ "\n")
