module PrintExtract where

-- pretty-printer generated by the BNF converter

import AbsExtract
import Char

-- the top-level printing method
printTree :: Print a => a -> String
printTree = render . prt 0

type Doc = [ShowS] -> [ShowS]

doc :: ShowS -> Doc
doc = (:)

render :: Doc -> String
render d = rend 0 (map ($ "") $ d []) "" where
  rend i ss = case ss of
    "["      :ts -> showChar '[' . rend i ts
    "("      :ts -> showChar '(' . rend i ts
    "{"      :ts -> showChar '{' . new (i+1) . rend (i+1) ts
    "}" : ";":ts -> new (i-1) . space "}" . showChar ';' . new (i-1) . rend (i-1) ts
    "}"      :ts -> new (i-1) . showChar '}' . new (i-1) . rend (i-1) ts
    ";"      :ts -> showChar ';' . new i . rend i ts
    t  : "," :ts -> showString t . space "," . rend i ts
    t  : ")" :ts -> showString t . showChar ')' . rend i ts
    t  : "]" :ts -> showString t . showChar ']' . rend i ts
    t        :ts -> space t . rend i ts
    _            -> id
  new i   = showChar '\n' . replicateS (2*i) (showChar ' ') . dropWhile isSpace
  space t = showString t . (\s -> if null s then "" else (' ':s))

parenth :: Doc -> Doc
parenth ss = doc (showChar '(') . ss . doc (showChar ')')

concatS :: [ShowS] -> ShowS
concatS = foldr (.) id

concatD :: [Doc] -> Doc
concatD = foldr (.) id

replicateS :: Int -> ShowS -> ShowS
replicateS n f = concatS (replicate n f)

-- the printer class does the job
class Print a where
  prt :: Int -> a -> Doc
  prtList :: [a] -> Doc
  prtList = concatD . map (prt 0)

instance Print a => Print [a] where
  prt _ = prtList

instance Print Char where
  prt _ s = doc (showChar '\'' . mkEsc '\'' s . showChar '\'')
  prtList s = doc (showChar '"' . concatS (map (mkEsc '"') s) . showChar '"')

mkEsc :: Char -> Char -> ShowS
mkEsc q s = case s of
  _ | s == q -> showChar '\\' . showChar s
  '\\'-> showString "\\\\"
  '\n' -> showString "\\n"
  '\t' -> showString "\\t"
  _ -> showChar s

prPrec :: Int -> Int -> Doc -> Doc
prPrec i j = if j<i then parenth else id


instance Print Integer where
  prt _ x = doc (shows x)


instance Print Double where
  prt _ x = doc (shows x)


instance Print Ident where
  prt _ (Ident i) = doc (showString i)



instance Print Grammar where
  prt i e = case e of
   Grammar defs -> prPrec i 0 (concatD [prt 0 defs])


instance Print Def where
  prt i e = case e of
   Paradigm id env patterns logic -> prPrec i 0 (concatD [doc (showString "paradigm") , prt 0 id , prt 0 env , doc (showString "=") , prt 0 patterns , doc (showString "{") , prt 0 logic , doc (showString "}")])
   ParadigmN id patterns logic -> prPrec i 0 (concatD [doc (showString "paradigm") , prt 0 id , doc (showString "=") , prt 0 patterns , doc (showString "{") , prt 0 logic , doc (showString "}")])
   Regdef id reg -> prPrec i 0 (concatD [doc (showString "regexp") , prt 0 id , doc (showString "=") , prt 0 reg])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])

instance Print Pattern where
  prt i e = case e of
   P items -> prPrec i 0 (concatD [prt 0 items])

  prtList es = case es of
   [x] -> (concatD [prt 0 x])
   x:xs -> (concatD [prt 0 x , prt 0 xs])

instance Print Env where
  prt i e = case e of
   E bindings -> prPrec i 0 (concatD [doc (showString "[") , prt 0 bindings , doc (showString "]")])


instance Print Item where
  prt i e = case e of
   C str -> prPrec i 0 (concatD [prt 0 str])
   Var id -> prPrec i 0 (concatD [prt 0 id])

  prtList es = case es of
   [x] -> (concatD [prt 0 x])
   x:xs -> (concatD [prt 0 x , doc (showString "+") , prt 0 xs])

instance Print Binding where
  prt i e = case e of
   Bind id reg -> prPrec i 0 (concatD [prt 0 id , doc (showString ":") , prt 0 reg])

  prtList es = case es of
   [] -> (concatD [])
   [x] -> (concatD [prt 0 x])
   x:xs -> (concatD [prt 0 x , doc (showString ",") , prt 0 xs])

instance Print Logic where
  prt i e = case e of
   Conj logic0 logic -> prPrec i 0 (concatD [prt 0 logic0 , doc (showString "&") , prt 1 logic])
   Disj logic0 logic -> prPrec i 0 (concatD [prt 0 logic0 , doc (showString "|") , prt 1 logic])
   Neg logic -> prPrec i 1 (concatD [doc (showString "~") , prt 1 logic])
   LPatt pattern -> prPrec i 1 (concatD [prt 0 pattern])


instance Print Reg where
  prt i e = case e of
   RAlt reg0 reg -> prPrec i 0 (concatD [prt 0 reg0 , doc (showString "|") , prt 1 reg])
   RMinus reg0 reg -> prPrec i 0 (concatD [prt 1 reg0 , doc (showString "-") , prt 1 reg])
   RSeq reg0 reg -> prPrec i 1 (concatD [prt 1 reg0 , prt 2 reg])
   RStar reg -> prPrec i 2 (concatD [prt 2 reg , doc (showString "*")])
   RPlus reg -> prPrec i 2 (concatD [prt 2 reg , doc (showString "+")])
   ROpt reg -> prPrec i 2 (concatD [prt 2 reg , doc (showString "?")])
   REps  -> prPrec i 2 (concatD [doc (showString "eps")])
   RChar c -> prPrec i 2 (concatD [prt 0 c])
   RAlts str -> prPrec i 2 (concatD [doc (showString "[") , prt 0 str , doc (showString "]")])
   RDigit  -> prPrec i 2 (concatD [doc (showString "digit")])
   RLetter  -> prPrec i 2 (concatD [doc (showString "letter")])
   RUpper  -> prPrec i 2 (concatD [doc (showString "upper")])
   RLower  -> prPrec i 2 (concatD [doc (showString "lower")])
   RAny  -> prPrec i 2 (concatD [doc (showString "char")])
   RStr str -> prPrec i 2 (concatD [prt 0 str])
   RegVar id -> prPrec i 2 (concatD [prt 0 id])



