module FMTrie (tcompile, SATrie, LexData(..), prLexData,
	       insert, emptyTrie,decompose,tlookup,isInTrie, 
               Trie, collapse, collapseN) where
    
import General
import Maybe(fromJust,fromMaybe)
import List(nub, inits, tails)
import qualified Trie

type Trie a = Trie.Trie Char a

data LexData = LD String String String String [String] String
 deriving Eq

instance Show LexData where
  show (LD stem n typ par inhs extra) = 
      unwords $ stem : ('(':n ++ ")") : typ : sp : par : (ex extra inhs)
   where sp = "-"
	 ex []     []     = []
         ex (x:xs) []     = sp : [(x:xs)]
         ex []     (y:ys) = sp : (y:ys)
         ex (x:xs) (y:ys) = sp : (x:xs) : sp : (y:ys)

prLexData :: LexData -> String
prLexData = show

-- * Trie with list values

emptyTrie :: Trie [a]
emptyTrie = Trie.empty

tcompile :: [(String, [a])] -> Trie [a]
tcompile = Trie.fromListWith (++) 

insert :: (String,[a]) -> Trie [a] -> Trie [a]
insert (k,v) = Trie.insertWith (++) k v

trieLookup :: Trie [a] -> String -> [a]
trieLookup tr s = fromMaybe [] (Trie.lookup s tr)

isInTrie :: Trie [a] -> String -> Bool
isInTrie tr s = Trie.member s tr

collapse :: Trie [a] -> [(String,[a])]
collapse = Trie.toList

collapseN :: Trie [a] -> ([(String,[a])],Int)
collapseN t = case Trie.toList t of
               xs -> (xs,length xs)

-- * Trie with values being a list of attribute, value pairs

type AttrTrie a = Trie [(Attr,a)]

type SATrie = AttrTrie LexData

tlookup :: SATrie -> String -> (String,[(Attr,String)])
tlookup t s = (s,[(a,show p) | (a,p) <- trieLookup t s])

decompose :: SATrie -> ([Attr] -> Bool) -> String -> [[(Attr,String)]]
decompose trie _  [] = []
decompose trie f sentence = 
    map (map (\(a,b) -> (a,prLexData b))) $ 
	concat $ map (legal trie f) $ deconstruct trie sentence

-- backtrack [(sentence,[])] trie

legal :: SATrie ->  ([Attr] -> Bool) -> [String] -> [[(Attr,LexData)]]
legal trie f input = removeInvalids attrValues
 where
  removeInvalids [] = [] -- Remove all invalid analysis
  removeInvalids (xs:xss)
   | f (map fst xs)           = xs : removeInvalids xss -- Sequence valid
   | otherwise                = removeInvalids xss
  flatten       [] = [[]] -- combine all analyses with all other analyses
  flatten (xs:xss) = [x:ys | x <- xs, ys <- res] 
      where res = flatten xss
  attrValues = flatten $ map (trieLookup trie) input

-- Improved decomposition

deconstruct :: AttrTrie a -> String -> [[String]]
deconstruct t [] = [[]]
deconstruct t s  = 
    concat [map (p:) (deconstruct t r) | (p,r) <- zip (inits s) (tails s), isInTrie t p]
